# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: turkish_treebanks/twt.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='turkish_treebanks/twt.proto',
  package='turkish_treebanks',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1bturkish_treebanks/twt.proto\x12\x11turkish_treebanks\"#\n\x03Tag\x12\x0e\n\x06\x63oarse\x18\x01 \x01(\t\x12\x0c\n\x04\x66ine\x18\x02 \x01(\t\"*\n\x07\x46\x65\x61ture\x12\x10\n\x08\x63\x61tegory\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xd3\x01\n\x05Token\x12\x0c\n\x04\x66orm\x18\x01 \x01(\t\x12\r\n\x05lemma\x18\x02 \x01(\t\x12#\n\x03tag\x18\x03 \x01(\x0b\x32\x16.turkish_treebanks.Tag\x12+\n\x07\x66\x65\x61ture\x18\x04 \x03(\x0b\x32\x1a.turkish_treebanks.Feature\x12\x0c\n\x04head\x18\x05 \x01(\x05\x12\x1b\n\x13\x64\x65pendency_relation\x18\x06 \x01(\t\x12\x30\n\x0cmisc_feature\x18\x07 \x03(\x0b\x32\x1a.turkish_treebanks.Feature\"V\n\x08Sentence\x12\x13\n\x0bsentence_id\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\'\n\x05token\x18\x03 \x03(\x0b\x32\x18.turkish_treebanks.Token'
)




_TAG = _descriptor.Descriptor(
  name='Tag',
  full_name='turkish_treebanks.Tag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='coarse', full_name='turkish_treebanks.Tag.coarse', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fine', full_name='turkish_treebanks.Tag.fine', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=85,
)


_FEATURE = _descriptor.Descriptor(
  name='Feature',
  full_name='turkish_treebanks.Feature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='category', full_name='turkish_treebanks.Feature.category', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='turkish_treebanks.Feature.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=87,
  serialized_end=129,
)


_TOKEN = _descriptor.Descriptor(
  name='Token',
  full_name='turkish_treebanks.Token',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='form', full_name='turkish_treebanks.Token.form', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lemma', full_name='turkish_treebanks.Token.lemma', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag', full_name='turkish_treebanks.Token.tag', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature', full_name='turkish_treebanks.Token.feature', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='head', full_name='turkish_treebanks.Token.head', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dependency_relation', full_name='turkish_treebanks.Token.dependency_relation', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='misc_feature', full_name='turkish_treebanks.Token.misc_feature', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=132,
  serialized_end=343,
)


_SENTENCE = _descriptor.Descriptor(
  name='Sentence',
  full_name='turkish_treebanks.Sentence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sentence_id', full_name='turkish_treebanks.Sentence.sentence_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text', full_name='turkish_treebanks.Sentence.text', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='token', full_name='turkish_treebanks.Sentence.token', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=345,
  serialized_end=431,
)

_TOKEN.fields_by_name['tag'].message_type = _TAG
_TOKEN.fields_by_name['feature'].message_type = _FEATURE
_TOKEN.fields_by_name['misc_feature'].message_type = _FEATURE
_SENTENCE.fields_by_name['token'].message_type = _TOKEN
DESCRIPTOR.message_types_by_name['Tag'] = _TAG
DESCRIPTOR.message_types_by_name['Feature'] = _FEATURE
DESCRIPTOR.message_types_by_name['Token'] = _TOKEN
DESCRIPTOR.message_types_by_name['Sentence'] = _SENTENCE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Tag = _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
  'DESCRIPTOR' : _TAG,
  '__module__' : 'turkish_treebanks.twt_pb2'
  # @@protoc_insertion_point(class_scope:turkish_treebanks.Tag)
  })
_sym_db.RegisterMessage(Tag)

Feature = _reflection.GeneratedProtocolMessageType('Feature', (_message.Message,), {
  'DESCRIPTOR' : _FEATURE,
  '__module__' : 'turkish_treebanks.twt_pb2'
  # @@protoc_insertion_point(class_scope:turkish_treebanks.Feature)
  })
_sym_db.RegisterMessage(Feature)

Token = _reflection.GeneratedProtocolMessageType('Token', (_message.Message,), {
  'DESCRIPTOR' : _TOKEN,
  '__module__' : 'turkish_treebanks.twt_pb2'
  # @@protoc_insertion_point(class_scope:turkish_treebanks.Token)
  })
_sym_db.RegisterMessage(Token)

Sentence = _reflection.GeneratedProtocolMessageType('Sentence', (_message.Message,), {
  'DESCRIPTOR' : _SENTENCE,
  '__module__' : 'turkish_treebanks.twt_pb2'
  # @@protoc_insertion_point(class_scope:turkish_treebanks.Sentence)
  })
_sym_db.RegisterMessage(Sentence)


# @@protoc_insertion_point(module_scope)



# coding=utf-8
# Copyright 2020 The Google Research Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Functions to read Turkish Web Treebank sentence annotations."""

import os
import pathlib
from typing import Generator, Iterable, List, Optional

from turkish_treebanks import twt_pb2

_Feature = twt_pb2.Feature
_Sentence = twt_pb2.Sentence
_Tag = twt_pb2.Tag
_Token = twt_pb2.Token

_ROOT_DIR = pathlib.Path(__file__).parent.parent
_DATA_DIR = os.path.join(_ROOT_DIR, "data")
_PATHS_BY_SECTION = {
    "web": os.path.join(_DATA_DIR, "web.conllu"),
    "wiki": os.path.join(_DATA_DIR, "wiki.conllu"),
}
_VALID_SPLIT_NAMES = [
    "train",
    "dev",
    "test",
]


def _whitespace_trimmed(string: str) -> str:
  """Strips any leading and trailing whitespace off from the string"""
  return string.lstrip().rstrip()


def _split_into_sentences(conll: str) -> List[str]:
  """Tokenizes contents of a CoNLL-U format file by sentence annotations."""
  return [_whitespace_trimmed(s) for s in conll.split("\n\n")]


def _split_into_lines(sentence: str) -> List[str]:
  """Tokenizes CoNLL-U format sentence annotation into lines."""
  return [_whitespace_trimmed(l) for l in sentence.split("\n")]


def _reconstruct_conll_from(sentences: Iterable[str]) -> str:
  """Reconstructs CoNLL-U format file content from sentence annotations."""
  return "\n\n".join(sentences)


def _read_sentences_from(path: str) -> Generator[str, None, None]:
  """Reads and yields sentences of a CoNLL-U format treebank file from the path.

  Args:
    path: path to a CoNLL-U format treebank file from which sentences will be
        read.

  Yields:
    Individual sentence annotations of a CoNLL-U format treebank file.
  """
  with open(path, "r", encoding="utf-8") as reader:
    yield from _split_into_sentences(reader.read())


def _sentence_is_in_split(sentence_index: int, split: str) -> bool:
  """Checks if sentence with given positional index is in the split.

  In order to deterministically sample train/dev/test sentences, this function
  assumes every 9th sentence of a treebank file belongs to the development
  set, and every 10th sentence of a treebank file belongs to the test set. All
  other sentences belong to the training set.

  Args:
    sentence_index: sequential index of the sentence in the source CoNLL-U
        format treebank file (assuming first sentence has index 0).
    split: treebank split (could be 'train', 'test', 'dev').

  Returns:
    True if sentence with given positional index belongs to the specified split.
    Otherwise, returns False.
  """
  if split == "train":
    return sentence_index % 10 < 8
  elif split == "dev":
    return sentence_index % 10 == 8
  elif split == "test":
    return sentence_index % 10 == 9
  else:
    return True


def _validate_sentence(sentence: str) -> None:
  """Checks if a CoNLL-U format sentence annotation is structurally wellformed.

  Args:
    sentence: a CoNNL-U format sentence annotation.

  Raises:
    ValueError: CoNNL-U format sentence annotation is illformed. It is either
        missing sentence id or text annotation, or one of the lines that ought
        to contain token annotations is structurally illformed.
  """
  lines = _split_into_lines(sentence)

  if len(lines) <= 2:
    raise ValueError(
        f"Expecting a sentence to be at least 3 lines in CoNLL-U format,"
        f" but found a {len(sentence)} line sentence annotation:\n{sentence}")

  if not lines[0].startswith("# sent_id = "):
    raise ValueError(
        f"First line of the CoNNL-U format sentence annotation does not have a"
        f" valid sentence id annotation:\n{sentence}")

  if not lines[1].startswith("# text = "):
    raise ValueError(
        f"Second line of the CoNNL-U format sentence annotation does not have a"
        f" valid sentence text annotation:\n{sentence}")

  def _validate_token_line(line: str) -> None:
    if len(line.split("\t")) != 10:
      raise ValueError(f"Illformed CoNNL-U format token annotation:\n{line}")

  for line in lines[3:]:
    _validate_token_line(line)


def _decompose_sentence(sentence: str) -> _Sentence:
  """Parses CoNLL-U format sentence annotation in a sentence protobuf.

  Args:
    sentence: a CoNNL-U format sentence annotation.

  Returns:
    Sentence protobuf that contains annotations for a sentence that is parsed
    from CoNLL-U format sentence annotation.
  """

  def _decompose_comment(line: str, prefix: str) -> None:
    """Decomposes a comment line of CoNNL-U format."""
    return line[len(prefix):]

  def _decompose_features(raw_features: str) -> Generator[_Feature, None, None]:
    """Parses CoNLL-U format features annotations into Feature objects."""
    category_value = (f.split("=") for f in raw_features.split("|") if f != "_")
    yield from (_Feature(category=n, value=v) for n, v in category_value)

  def _decompose_token(line: str) -> _Token:
    """Parses CoNLL-U format token annotations into Token objects."""
    column = line.split("\t")
    return _Token(
        form=column[1],
        lemma=column[2],
        tag=_Tag(coarse=column[3], fine=column[4]),
        feature=_decompose_features(column[5]),
        head=int(column[6]),
        dependency_relation=column[7],
        misc_feature=_decompose_features(column[9]),
    )

  lines = _split_into_lines(sentence)
  return _Sentence(
      sentence_id=_decompose_comment(lines[0], "# sent_id = "),
      text=_decompose_comment(lines[1], "# text = "),
      token=tuple(_decompose_token(l) for l in lines[2:]),
  )


def as_conllu(section: Optional[str] = None,
              split: Optional[str] = None) -> str:
  """Reads sentence annotations of Turkish Web Treebank in CoNLL-U format.

  Args:
    section: optional, section of Turkish Web Treebank whose sentence
        annotations will be read (could be either 'web' or 'wiki'). If
        unspecified sentence annotations from web and Wikipedia sections will
        be read.
    split: optional, treebank split whose sentence annotations will be read
        (could be 'train', 'test', 'dev'). If unspecified sentence annotations
        from all three splits will be read.

  Raises:
    ValueError: invalid section name or split specifier, or source treebank
        files from which the sentence annotations are read is not valid with
        respect to the CoNLL-U format.

  Returns:
    Sentence annotations for the specified treebank and split in CoNNL-U
    treebank file format.
  """
  if section and section not in _PATHS_BY_SECTION:
    raise ValueError(f"Invalid section name '{section}'."
                     f" It can only be one of: 'web', 'wiki'.")

  if split and split not in _VALID_SPLIT_NAMES:
    raise ValueError(f"Invalid split specifier '{split}'."
                     f" It can only be one of: 'train', 'dev', 'test'")

  if section:
    paths = (p for s, p in _PATHS_BY_SECTION.items() if s == section)
  else:
    paths = _PATHS_BY_SECTION.values()

  def _filtered_sentences() -> Generator[str, None, None]:
    for path in sorted(paths):
      for index, sentence in enumerate(_read_sentences_from(path)):
        if _sentence_is_in_split(index, split):
          _validate_sentence(sentence)
          yield sentence

  return _reconstruct_conll_from(_filtered_sentences())


def sentences(section: Optional[str] = None,
              split: Optional[str] = None) -> Generator[_Sentence, None, None]:
  """Reads and yields sentences of Turkish Web Treebank as sentence protobufs.

  Args:
    section: optional, section of Turkish Web Treebank whose sentence
        annotations will be read (could be either 'web' or 'wiki'). If
        unspecified sentence annotations from both web and Wikipedia sections
        will be read.
    split: optional, treebank split whose sentence annotations will be read
        (could be 'train', 'test', 'dev'). If unspecified sentence annotations
        from all three splits will be read.

  Raises:
    ValueError: invalid section name or split specifier, or source treebank
        files from which the sentence annotations are read is not valid with
        respect to the CoNLL-U format.

  Yields:
    Sentence protobufs which contain annotations for the specified treebank
    section and split.
  """
  for sentence in _split_into_sentences(as_conllu(section, split)):
    yield _decompose_sentence(sentence)
